import 'dart:async';
import 'dart:io';
import '../../utils/version.dart';

class GoConsole {
  final String packageName;
  final String projectName;
  final Version version;
  GoConsole({this.packageName, this.projectName, this.version});

  Future<void> done() async {
    // git mod
    await _createGitignore();
    await _createMod();

    // utils
    await _createVersion();
    await _createConfigure();
    await _createConfigureFile();
    await _createLogger();
    await _createUtils();

    // grpc
    await _createPB();
    await _createRegisterGRPC();

    // cmd
    await _createDaemon();
    await _createCommand();
    await _createMain();

    // vscode
    await _createSettings();
    await _createSettings();
    await _createTasks();
    await _createLaunch();

    // bash scripts
    await _createGRPC();
    await _createBuild();
  }

  Future<void> _createGRPC() async {
    final file = File("grpc.sh");
    if (await _existFile(file)) {
      return;
    }
    await file.writeAsString(r'''
#!/bin/bash
#Program:
#       自動 查找 編譯 google's grpc 代碼
#History:
#       2018-09-18 king first release support golang/c++
#       2019-02-24 king support java
#       2019-02-28 king support dart
#Email:
#       zuiwuchang@gmail.com

# 顯示幫助信息
function ShowHelp(){
    echo "help              : show help"
    echo "go root output    : build src/*.proto for golang" 
    echo "dart root output  : build src/*.proto for dartlang"
    echo "cpp root output   : build src/*.proto for c++"
    echo "java root output  : build src/*.proto for java"
}

# 遞歸 查詢所有的 檔案夾/包
# $1 根本目錄
function find_package(){
    _DIRS=""
    files=`find $1 -type d`
    ok=$?
    if [[ $ok != 0 ]];then
        exit $ok
    fi

    for str in $files
    do
        str=${str#$1}
        str=${str#.}
        str=${str#/}
        if [ "$str" ];then
            _DIRS="$_DIRS $str"
        fi
    done
}
# 查找 proto 檔案
# $1 根本目錄
# $2 包路徑
function find_proto(){
    _SOURCES=""

    files=`find $1/$2 -maxdepth 1 -name *.proto -type f`
    ok=$?
    if [[ $ok != 0 ]];then
        exit $ok
    fi

    for str in $files
    do
        str=${str#$1}
        str=${str#.}
        str=${str#/}
        if [ "$str" ];then
            _SOURCES="$_SOURCES $str"
        fi
    done
}

function print_source(){
    if [ "$2" ];then
        for str in $2
        do
            str=${str#$1}
            str=${str#.}
            str=${str#/}
            echo "   $str"
        done
    else
        echo "  warning : not found any source"
    fi
    echo "}"
    echo
}
function check_params(){
    if [ ! "$2" ];then
        echo "need param root directory"
        echo "exmaple : grpc.sh $1 proto protocol/$1"
        exit 1
    fi
    if [ ! "$3" ];then
        echo "need param output"
        echo "exmaple : grpc.sh $1 proto protocol/$1"
        exit 1
    fi

    if [ ! -d "$3" ];then
        echo "directory not exist : $3"
        exit 1
    fi
}
# 自動 查找 並編譯 grpc 到 go 代碼
# $1 protoc 根目錄
# $2 輸出目錄
function BuildGo(){
    check_params go $1 $2
    root=$1
    out=$2

    find_package $root
    for dir in $_DIRS
    do
        echo
        echo "package $dir {"
        find_proto $root $dir
        print_source $dir $_SOURCES
        if [ "$_SOURCES" ];then
            echo "protoc -I $root --go_out=plugins=grpc:$out $_SOURCES"
            protoc -I $root --go_out=plugins=grpc:$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi
        fi
    done
}

# 自動 查找 並編譯 grpc 到 dart 代碼
# $1 protoc 根目錄
# $2 輸出目錄
function BuildDart(){
    check_params dart $1 $2
    root=$1
    out=$2

    find_package $root
    for dir in $_DIRS
    do
        echo
        echo "package $dir {"
        find_proto $root $dir
        print_source $dir $_SOURCES
        if [ "$_SOURCES" ];then
            echo "protoc -I $root --dart_out=grpc:$out $_SOURCES"
            protoc -I $root --dart_out=grpc:$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi
        fi
    done
}

# 自動 查找 並編譯 grpc 到 c++ 代碼
# $1 protoc 根目錄
# $2 輸出目錄
function BuildCpp(){
    check_params cpp $1 $2
    root=$1
    out=$2

    GrpcCppPlugin=`which grpc_cpp_plugin`
    if [ -f "$GrpcCppPlugin".exe ];then
        GrpcCppPlugin="$GrpcCppPlugin".exe
    fi

    if [ ! -f "$GrpcCppPlugin" ];then
        echo grpc_cpp_plugin not found
        exit 1
    fi

    find_package $root
    for dir in $_DIRS
    do
        echo
        echo "package $dir {"
        find_proto $root $dir
        print_source $dir $_SOURCES
        if [ "$_SOURCES" ];then
            # pb
            echo "protoc -I $root --cpp_out=$out $_SOURCES"
            protoc -I $root --cpp_out=$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi

            # grpc
            echo "protoc -I $root --plugin=protoc-gen-grpc=$GrpcCppPlugin --grpc_out=$out $_SOURCES"
            protoc -I $root --plugin=protoc-gen-grpc=$GrpcCppPlugin --grpc_out=$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi
        fi
    done
}
# 自動 查找 並編譯 grpc 到 java 代碼
# $1 protoc 根目錄
# $2 輸出目錄
function BuildJava(){
    check_params java $1 $2
    root=$1
    out=$2

    find_package $root
    for dir in $_DIRS
    do
        echo
        echo "package $dir {"
        find_proto $root $dir
        print_source $dir $_SOURCES
        if [ "$_SOURCES" ];then
            # pb
            echo "protoc -I $root --java_out=$out $_SOURCES"
            protoc -I $root --java_out=$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi

            # grpc
            echo "protoc --plugin=protoc-gen-grpc-java --proto_path=$root --grpc-java_out=$out $_SOURCES"
            protoc --plugin=protoc-gen-grpc-java --proto_path=$root --grpc-java_out=$out $_SOURCES
            ok=$?
            if [[ $ok != 0 ]];then
                exit $ok
            fi
        fi
    done
}

ok=0
case $1 in
    go)
        BuildGo $2 $3
        ok=$?
    ;;

    dart)
        BuildDart $2 $3
        ok=$?
    ;;

    cpp)
        BuildCpp $2 $3
        ok=$?
    ;;

    java)
        BuildJava $2 $3
        ok=$?
    ;;

    *)
        ShowHelp
        ok=$?
    ;;
esac
exit $ok
''');

    if (Platform.isLinux || Platform.isMacOS) {
      Process.runSync("chmod", ["a+x", "grpc.sh"]);
    }
  }

  Future<void> _createBuild() async {
    final file = File("build.sh");
    if (await _existFile(file)) {
      return;
    }

    String grpcOutput = "";
    for (var i = 0; i < packageName.split("/").length; i++) {
      grpcOutput += "../";
    }
    await file.writeAsString('''
#!/bin/bash
#Program:
#       golang build scripts by generate tools
#
#       https://pub.dartlang.org/packages/generate
#Email:
#       zuiwuchang@gmail.com
function check(){
	if [ "\$1" != 0 ] ;then
	exit \$1
	fi
}
function ShowHelp(){
	echo "help                 : show help"
	echo "l/linux   [t/tar]    : build for linux"
	echo "d/darwin  [t/tar]    : build for darwin"
	echo "w/windows [t/tar]    : build for windows"
	echo "g/grpc               : build for proto to grpc"
}

function goGRPC(){
  dir=\$(cd \$(dirname \$BASH_SOURCE)/$grpcOutput && pwd)
  if [[ "\$OSTYPE" == "msys" ]]; then
    output=\${dir:1:1}:\${dir:2}/
  else
    output=\$dir/
  fi
  ./grpc.sh go pb/ \$output
}
case \$1 in
	g|grpc)
    goGRPC
	;;

	l|linux)
		export GOOS=linux

		echo go build -ldflags "-s -w" -o bin/$projectName
		go build -ldflags "-s -w" -o bin/$projectName
		check \$?

		if [[ \$2 == tar || \$2 == t ]]; then
			dst=linux.amd64.tar.gz
			if [[ \$GOARCH == 386 ]];then
				dst=linux.386.tar.gz
			fi
			cd bin && tar -zcvf \$dst $projectName
		fi
	;;

	d|darwin)
		export GOOS=darwin

		echo go build -ldflags "-s -w" -o bin/$projectName
		go build -ldflags "-s -w" -o bin/$projectName
		check \$?

		if [[ \$2 == tar || \$2 == t ]]; then
			dst=darwin.amd64.tar.gz
			if [[ \$GOARCH == 386 ]];then
				dst=darwin.386.tar.gz
			fi
			cd bin && tar -zcvf \$dst $projectName
		fi
	;;

	w|windows)
		export GOOS=windows

		echo go build -ldflags "-s -w" -o bin/$projectName.exe
		go build -ldflags "-s -w" -o bin/$projectName.exe
		check \$?

		if [[ \$2 == tar || \$2 == t ]]; then
			dst=windows.amd64.tar.gz
			if [[ \$GOARCH == 386 ]];then
				dst=windows.386.tar.gz
			fi
			cd bin && tar -zcvf \$dst $projectName.exe
		fi
	;;

	*)
		ShowHelp
	;;
esac
''');

    if (Platform.isLinux || Platform.isMacOS) {
      Process.runSync("chmod", ["a+x", "build.sh"]);
    }
  }

  Future<void> _createLaunch() async {
    final file = File(".vscode/launch.json");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
{
    // Use IntelliSense to learn about possible attributes.
    // Hover to view descriptions of existing attributes.
    // For more information, visit: https://go.microsoft.com/fwlink/?linkid=830387
    "version": "0.2.0",
    "configurations": [
        {
            "name": "Launch",
            "type": "go",
            "request": "launch",
            "mode": "exec",
            "program": "\${workspaceRoot}/bin/${projectName}d",
            "windows": {
                "program": "\${workspaceRoot}/bin/${projectName}d.exe",
            },
            "env": {},
            "args": [
                "-v"
            ]
        }
    ]
}
''');
  }

  Future<void> _createTasks() async {
    final file = File(".vscode/tasks.json");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
{
    // See https://go.microsoft.com/fwlink/?LinkId=733558
    // for the documentation about the tasks.json format
    "version": "2.0.0",
    "tasks": [
        {
            "label": "build debug",
            "type": "shell",
            "command": "go build -o bin/${projectName}d",
            "windows": {
                "command": "go build -o bin/${projectName}d.exe",
            },
            "options": {
                "cwd": "\${workspaceRoot}"
            },
            "group": {
                "kind": "build",
                "isDefault": true
            },
            "problemMatcher": "\$go",
        },
        {
            "label": "build release",
            "type": "shell",
            "command": "go build -ldflags '-s -w' -o bin/$projectName",
            "windows": {
                "command": "go build -ldflags '-s -w' -o bin/$projectName.exe",
            },
            "options": {
                "cwd": "\${workspaceRoot}"
            },
            "group": {
                "kind": "build",
                "isDefault": true
            },
            "problemMatcher": "\$go",
        },
    ]
}
''');
  }

  Future<void> _createSettings() async {
    final file = File(".vscode/settings.json");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
{
    "editor.formatOnSave": true,
    "files.exclude": {
        "**/.classpath": true,
        "**/.project": true,
        "**/.settings": true,
        "**/.factorypath": true,
        "go.sum": true,
    },
    "files.associations": {
        "*.json": "jsonc"
    },
}
''');
  }

  Future<void> _createUtils() async {
    final file = File("utils/func.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package utils

import (
	"log"
	"os"
	"os/exec"
	"path/filepath"
)

var _BasePath string

// BasePath return executable file directory
func BasePath() string {
	if _BasePath != "" {
		return _BasePath
	}
	filename, e := exec.LookPath(os.Args[0])
	if e != nil {
		log.Fatalln(e)
	}

	filename, e = filepath.Abs(filename)
	if e != nil {
		log.Fatalln(e)
	}
	_BasePath = filepath.Dir(filename)
	return _BasePath
}

// Abs Use bashPath as the working directory to return the absolute path
func Abs(bashPath, path string) string {
	if filepath.IsAbs(path) {
		path = filepath.Clean(path)
	} else {
		path = filepath.Clean(bashPath + "/" + path)
	}
	return path
}
''');
  }

  Future<void> _createLogger() async {
    final file = File("logger/init.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package logger

import (
	"log"
	"path/filepath"
	"strings"

	logger "gitlab.com/king011/king-go/log/logger.zap"
	"go.uber.org/zap"
)

// Logger logger single
var Logger logger.Logger

// JoinFields .
var JoinFields = logger.JoinFields

// Fields .
var Fields = logger.Fields

// Init init logger
func Init(basePath string, options *logger.Options) (e error) {
	// format
	options.Filename = strings.TrimSpace(options.Filename)
	if options.Filename != "" {
		if !filepath.IsAbs(options.Filename) {
			options.Filename = filepath.Clean(basePath + "/" + options.Filename)
		}
	}
	var zapOptions []zap.Option
	if options.Caller {
		zapOptions = append(zapOptions, zap.AddCaller())
	}

	// new logger
	l := logger.New(options, zapOptions...)

	// run http
	if options.HTTP != "" {
		errHTTP := l.StartHTTP()
		if errHTTP == nil {
			if l.OutFile() {
				log.Println("zap http running", options.HTTP)
			}
			l.Info("zap http",
				zap.String("running", options.HTTP),
			)
		} else {
			if l.OutFile() {
				log.Println("zap http running", errHTTP)
			}
			l.Warn("zap http",
				zap.Error(errHTTP),
			)
		}
	}
	// Attach
	Logger.Attach(l)
	return
}
''');
  }

  Future<void> _createConfigure() async {
    final file = File("configure/Configure.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package configure

import (
	"encoding/json"
	"io/ioutil"
	"path/filepath"
	"strings"
	"github.com/google/go-jsonnet"
	logger "gitlab.com/king011/king-go/log/logger.zap"
	"$packageName/utils"
)

// Configure global configure
type Configure struct {
	Logger logger.Options
	GRPC   GRPC
}

// Format format global configure
func (c *Configure) Format(basePath string) (e error) {
	e = c.GRPC.Format(basePath)
	if e != nil {
		return
	}
	return
}
func (c *Configure) String() string {
	if c == nil {
		return "nil"
	}
	b, e := json.MarshalIndent(c, "", "	")
	if e != nil {
		return e.Error()
	}
	return string(b)
}

var _Configure Configure

// Single single Configure
func Single() *Configure {
	return &_Configure
}

// Load load configure file
func (c *Configure) Load(filename string) (e error) {
	var b []byte
	b, e = ioutil.ReadFile(filename)
	if e != nil {
		return
	}
	vm := jsonnet.MakeVM()
	var jsonStr string
	jsonStr, e = vm.EvaluateSnippet("", string(b))
	if e != nil {
		return
	}
	b = []byte(jsonStr)
	e = json.Unmarshal(b, c)
	if e != nil {
		return
	}
	return
}

// GRPC configure
type GRPC struct {
	// server listen addr
	Addr string

	// x509 Cert
	CertFile string
	// x509 Key
	KeyFile string
}

// Format format grpc configure
func (c *GRPC) Format(basePath string) (e error) {
	c.CertFile = strings.TrimSpace(c.CertFile)
	c.KeyFile = strings.TrimSpace(c.KeyFile)

	if c.CertFile != "" {
		if filepath.IsAbs(c.CertFile) {
			c.CertFile = filepath.Clean(c.CertFile)
		} else {
			c.CertFile = utils.Abs(basePath, c.CertFile)
		}
	}

	if c.KeyFile != "" {
		if filepath.IsAbs(c.KeyFile) {
			c.KeyFile = filepath.Clean(c.KeyFile)
		} else {
			c.KeyFile = utils.Abs(basePath, c.KeyFile)
		}
	}
	return
}

// H2 return turn if use h2
func (c *GRPC) H2() bool {
	return c.CertFile != "" && c.KeyFile != ""
}

// H2C return !H2
func (c *GRPC) H2C() bool {
	return c.CertFile == "" || c.KeyFile == ""
}
''');
  }

  Future<void> _createConfigureFile() async {
    final file = File("bin/$projectName.jsonnet");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
{
	GRPC:{
		// server listen addr
		Addr:"127.0.0.1:61000",

		// x509 Cert
		//CertFile:"test.crt",

		// x509 Key
		//KeyFile:"test.key",
	},
	Logger:{
		// zap http
		//HTTP:"localhost:20000",
		// log name
		//Filename:"logs/kc_serverlist.log",
		// MB
		MaxSize:    100, 
		// number of files
		MaxBackups: 3,
		// day
		MaxAge:     28,
		// level : debug info warn error dpanic panic fatal
		Level :"debug",
	},
}
''');
  }

  Future<void> _createVersion() async {
    final file = File("version/version.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package version

// Tag .
const Tag = "${version.tag}";
// Commit .
const Commit = "${version.commit}";
// Date .
const Date = "${version.datetime}";
''');
  }

  Future<void> _createDaemon() async {
    final file = File("cmd/daemon/daemon.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package daemon

import (
	"$packageName/configure"
	"$packageName/logger"
	"log"
	"net"
	"os"

	"go.uber.org/zap"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials"
	"google.golang.org/grpc/reflection"
)

// Run run server
func Run(cnf *configure.Configure) {
	runGRPC(&cnf.GRPC)
}
func runGRPC(cnf *configure.GRPC) {
	listen, e := net.Listen("tcp", cnf.Addr)
	if e != nil {
		if ce := logger.Logger.Check(zap.ErrorLevel, "grpc.Listen"); ce != nil {
			ce.Write(zap.Error(e))
		}
		os.Exit(1)
		return
	}
	// new server
	var opt []grpc.ServerOption
	if cnf.H2() {
		// log
		if ce := logger.Logger.Check(zap.InfoLevel, "h2 work"); ce != nil {
			ce.Write(zap.String("addr", cnf.Addr))
		}
		if !logger.Logger.OutConsole() {
			log.Println("h2 work at", cnf.Addr)
		}
		// opt
		creds, e := credentials.NewServerTLSFromFile(cnf.CertFile, cnf.KeyFile)
		if e != nil {
			if ce := logger.Logger.Check(zap.ErrorLevel, "grpc.Credentials"); ce != nil {
				ce.Write(zap.Error(e))
			}
			os.Exit(1)
			return
		}
		opt = append(opt,
			grpc.Creds(creds),
		)
	} else {
		// log
		if ce := logger.Logger.Check(zap.InfoLevel, "h2c work"); ce != nil {
			ce.Write(zap.String("addr", cnf.Addr))
		}
		if !logger.Logger.OutConsole() {
			log.Println("h2c work at", cnf.Addr)
		}
		// opt
	}
	server := grpc.NewServer(opt...)

	// register
	registerGRPC(server)
	reflection.Register(server)
	// run grpc
	if e := server.Serve(listen); e != nil {
		if ce := logger.Logger.Check(zap.ErrorLevel, "grpc.Serve"); ce != nil {
			ce.Write(zap.Error(e))
		}
		os.Exit(1)
		return
	}
}
''');
  }

  Future<void> _createCommand() async {
    final file = File("cmd/init.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package cmd

import (
	"fmt"
	"log"
	"runtime"

	"$packageName/configure"
	"$packageName/cmd/daemon"
	"$packageName/logger"
	"$packageName/utils"
	"$packageName/version"

	"github.com/spf13/cobra"
)

const (
	// App .
	App = "$projectName"
)

var v bool
var rootCmd = &cobra.Command{
	Use:   App,
	Short: "generate tools create $projectName",
	Run: func(cmd *cobra.Command, args []string) {
		if v {
			fmt.Println(runtime.GOOS, runtime.GOARCH, runtime.Version())
			fmt.Println(version.Tag)
			fmt.Println(version.Commit)
			fmt.Println(version.Date)
		} else {
			fmt.Println(App)
			fmt.Println(runtime.GOOS, runtime.GOARCH, runtime.Version())
			fmt.Println(version.Tag)
			fmt.Println(version.Commit)
			fmt.Println(version.Date)
			fmt.Printf(`Use "%v --help" for more information about this program.
`, App)
		}
	},
}

func init() {
	flags := rootCmd.Flags()
	flags.BoolVarP(&v,
		"version",
		"v",
		false,
		"show version",
	)
	initDaemon()
}
func initDaemon() {
	var filename string
	basePaht := utils.BasePath()
	cmd := &cobra.Command{
		Use:   "daemon",
		Short: "run as daemon",
		Run: func(cmd *cobra.Command, args []string) {
			// load configure
			cnf := configure.Single()
			e := cnf.Load(filename)
			if e != nil {
				log.Fatalln(e)
			}
			e = cnf.Format(basePaht)
			if e != nil {
				log.Fatalln(e)
			}

			// init logger
			e = logger.Init(basePaht, &cnf.Logger)
			if e != nil {
				log.Fatalln(e)
			}

			// run
			daemon.Run(cnf)
		},
	}
	flags := cmd.Flags()
	flags.StringVarP(&filename, "config",
		"c",
		utils.Abs(basePaht, "$projectName.jsonnet"),
		"configure file",
	)
	rootCmd.AddCommand(cmd)
}

// Execute run command
func Execute() error {
	return rootCmd.Execute()
}
''');
  }

  Future<void> _createMod() async {
    final file = File("go.mod");
    if (await _existFile(file)) {
      return;
    }
    await file.writeAsString('''
module $packageName
''');
  }

  Future<void> _createGitignore() async {
    final file = File(".gitignore");
    if (await _existFile(file)) {
      return;
    }
    await file.writeAsString('''
# Binaries for programs and plugins
*.exe
*.exe~
*.dll
*.so
*.dylib

# Test binary, build with `go test -c`
*.test

# Output of the go coverage tool, specifically when used with LiteIDE
*.out

/vendor/*
/protocol/*

/$projectName
/${projectName}d
/bin/$projectName
/bin/${projectName}d
*.tar.gz
''');
  }

  Future<void> _createMain() async {
    final file = File("main.go");
    if (await _existFile(file)) {
      return;
    }
    await file.writeAsString('''
package main

import (
	"log"
	"$packageName/cmd"
)

func main() {
	if e := cmd.Execute(); e != nil {
		log.Fatalln(e)
	}
}
''');
  }

  Future<bool> _existFile(File file) async {
    final stat = await file.stat();
    if (stat.type == FileSystemEntityType.notFound) {
      return false;
    }

    if (stat.type == FileSystemEntityType.directory) {
      throw StateError("main.go is a directory");
    }
    return true;
  }

  Future<void> _createRegisterGRPC() async {
    final file = File("cmd/daemon/grpc.go");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
package daemon

import (
	"context"
	grpc_example "$packageName/protocol/example"

	"google.golang.org/grpc"
)

func registerGRPC(server *grpc.Server) {
	grpc_example.RegisterServerServer(server, _Example{})
}

type _Example struct {
}

func (_Example) Ping(ctx context.Context, request *grpc_example.PingRequest) (response *grpc_example.PingResponse, e error) {
	response = &grpc_example.PingResponse{}
	return
}
''');
  }

  Future<void> _createPB() async {
    final file = File("pb/$packageName/protocol/example/example.proto");
    if (await _existFile(file)) {
      return;
    }
    await file.create(recursive: true);
    await file.writeAsString('''
syntax = "proto3";
package generate_$projectName.example;

service Server {
    rpc Ping (PingRequest) returns (PingResponse);
}

message PingRequest{
}
message PingResponse{
}
''');
  }
}
